/*
 *	File Name: tunerCard.cpp
 *	Function: Code to control radio tuner card via the kernel driver
 *
 *   	This program is free software; you can redistribute it and/or modify
 *   	it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *   	This program is distributed in the hope that it will be useful,
 *   	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   	GNU General Public License for more details.
 *
 *   	You should have received a copy of the GNU General Public License
 *   	along with this program; if not, write to the Free Software
 *   	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *	Author: Heng Yiak Por (alleycat@pacific.net.sg)
 *	Date of Release:  13 Apr 2000
 *	Version: 1.0.0
 *
 *	Bugs: None
 *
 */


#include <fstream.h>
#include <iostream.h>
#include "radioConfig.h"
#include <sys/timeb.h>
#include <time.h>
#include <qt/qdatetime.h>

#define RADIOCARD 	"/dev/pcr"
#define IF 		10700
#define FINESTEP 	50
#define SCANSTEP	50
#define DELAY		500

class tunerCard
{

	private:
		char data[4], muted, progNo;
		radioConfig *config;
		
			
		float	convertToFreq(char *);
		void 	convertToBin(float, char *);
		void	readCard(char *);
		void 	writeCard(char *);
		void 	timeDelay(void);

	public:
	
		tunerCard(radioConfig *);
		float	showFreq(void);
		void	setFreq(float);
		int		showStereo(void);
		void 	doRead(void);
		void	fineUp(void);
		void	fineDown(void);
		void	scanUp(void);
		void	scanDown(void);
		void	stereoToggle(void);
		void	setMute(void);
		void 	clearMute(void);
		char	isMuted(void);
		void	refreshData(void);
		void	loadProg(unsigned int);
		void	storeProg(unsigned int);
		int		ampOnBoard(void);
		int		isTuned(void);
		int		showApproxFreq(void);
		int		showProgNo(void);
		
};

/* isMuted function returns a 1 when FMR2 is forced muted */

char tunerCard::isMuted(void)
{
	return muted;
}

tunerCard::tunerCard(radioConfig *ptr)
{
	ofstream infile;

	infile.open(RADIOCARD, ios::in | ios::bin | ios::nocreate);
	
	if(!infile)
	{
		cout << "PCR Device not found....\nExiting Application....\n";
		exit(1);
	}
	
	infile.close();	
	
	config=ptr;
	
	doRead();
}

/* showFreq returns the value of the frequency currently loaded into FMR2 */

float tunerCard::showFreq(void)
{
		
	return (convertToFreq(data));
		
}

/* showApprox function is used for the LCD display, returns the rounded off value of the frequency cuurently in FMR2
   THE ACTUAL LOADED VALUE OF THE FREQUENCY IS TOO PRECISE TO BE SHOWN ON THE LCD SCREEN */

int tunerCard::showApproxFreq(void)
{
		int tmBuffer= (int) showFreq()/10;
		
		int tmp1 = tmBuffer%100;
		int tmp4 = tmBuffer/100;
		int tmp2 = tmp1 % 10;
		int tmp3 = tmp1 / 10;
		
		if((tmp2 <5) && (tmp2 != 0))
			tmp2 = 5;
		else if(tmp2 > 5)
		{
			tmp2 = 0;
			tmp3++;
			tmp3 %=10;
			if(tmp3 == 0)
				tmp4++;
		}
		
		tmBuffer= tmp4*100 + tmp3 * 10 + tmp2;
		
		return tmBuffer;
}

void tunerCard::setFreq(float value)
{
	char dataOut[4];
	
	convertToBin(value, dataOut);
	
	dataOut[1]=0;  /* Do not mute!*/
	dataOut[0]=0;  /* Do not start search !*/
		
	writeCard(dataOut);
	muted=0;
}

/* showStereo function returns a value of 1 when radio signal is mono. */

int tunerCard::showStereo(void)
{
	return (data[0] & 0x01);
}

/* ampOnBoard function returns a value of 1 when there is an on-Board amplifier on the FMR2 */

int tunerCard::ampOnBoard(void)
{
	return (data[0] & 0x04);
}

/* isTuned Function returns a value of 0 when the FMR2 is locked to a radio carrier */

int tunerCard::isTuned(void)
{
	return (data[0] & 0x02);
}

/* convertToFreq function converts the binary data from FMR2 to floating point value of the frequency divider ratio.
   buffer is an char array of 4 elements for the binary value. value is the floating point value of the
   the output frequency divider ratio.
*/


float tunerCard::convertToFreq(char *buffer)
{
	unsigned int frequencyBin, i;
	float frequency, freqStart;
	
	frequencyBin=buffer[3]&0xff;
	frequencyBin += (buffer[2] << 8) & 0xff00;

	for(i=0, freqStart = 12.5, frequency = 0; 
		i < 14; 
			i++, freqStart *= 2, frequencyBin >>= 1)
	{
		if(frequencyBin & 0x01 == 0x01)
		{
			frequency += freqStart;
		}
	}

	frequency -= IF; /* Compensate for IF */
	
	return frequency;

}

/* convertBin function converts the floating point value of the frequency divider ratio to binary form for input to the FMR2
   buffer is an char array of 4 elements for the converted binary value to be written to. value is the floating point value of the
   the input frequency divider ratio.
*/

void tunerCard::convertToBin(float value, char *buffer)
{
	unsigned int mask;
	int i;
	float startFreq;
	
	value+=10700;  /* Compensate for IF */
	
	for(i=13, startFreq = 102400, mask=0;
		 i >= 0; 
		 	i--, startFreq /= 2)
	{
		if((value - startFreq) >= 0)
		{
			mask |= (0x01) << i;
			value -= startFreq;
		}
		
	}
		
	buffer[3] = mask&0xff;
	buffer[2]= (mask&0xff00) >> 8;	
}

/* writeCard function: Writes the FMR2 device.
 * buffer is a pointer to an char array of 4 elements containing the data to be passed to the FMR2
*/

void tunerCard::writeCard(char *buffer)
{
	ofstream outfile;


	outfile.open(RADIOCARD, ios::out | ios::bin);
	outfile.write(buffer, 4);
	outfile.close();

}

/* readCard function: Reads the FMR2 device.
 * buffer is a pointer to an char array of 4 elements containing the data to be read from the FMR2
*/

void tunerCard::readCard(char *buffer)
{
	ifstream infile;

	infile.open(RADIOCARD, ios::in | ios::bin);
	infile.read(buffer, sizeof(*buffer)+5);
	infile.close();

}

void tunerCard::doRead(void)
{
	 readCard(data);
}


/* ScanUp function scan up the FM frequency spectrum */

void tunerCard::fineUp(void)
{
	doRead();
	
	float	tempFreq=convertToFreq(data);

	if(tempFreq < 108000)
		setFreq(tempFreq + FINESTEP);
}

/* FineDown function scan fine down the FM frequency spectrum */

void tunerCard::fineDown(void)
{
	doRead();

	float	tempFreq=convertToFreq(data);

	if(tempFreq > 88000)
		setFreq(tempFreq - FINESTEP);
}

/* Scanup function scan up the FM frequency spectrum */

void tunerCard::scanUp(void)
{

	char	temp[4];
	float	tempFreq=convertToFreq(data);
	int	cycle = 1;

	if(tempFreq < 108000)
	{
		doRead();
	
		int tempCmp1 = showApproxFreq();
		
		while(showApproxFreq() == tempCmp1)
		{
	
			while((isTuned() == 0) && (tempFreq < 108000))
			{
	
				tempFreq= tempFreq + cycle * SCANSTEP;
				setFreq(tempFreq);
				cycle++;

				timeDelay();
				doRead();
			}
		
			if(tempFreq < 108000)
			{
				convertToBin(tempFreq, temp);
	
				temp[1]=0x80 | (config->showSearchStr() & 0x03);
				temp[0]=0x01;	
	
				writeCard(temp);
	
				timeDelay();
				
				doRead();
			}
		}

	}
}

/* ScanDown function scan down the FM frequency spectrum */

void tunerCard::scanDown(void)
{

	char	temp[4];
	float	tempFreq=convertToFreq(data);
	
	int 	cycle=1;

	if(tempFreq > 88000)
	{
		doRead();

		int tempCmp1 = showApproxFreq();
		
		while(showApproxFreq() == tempCmp1)
		{
		
			while((isTuned() == 0) && (tempFreq > 88000))
			{
		
				tempFreq=tempFreq - cycle * SCANSTEP;
				setFreq(tempFreq);
				cycle++;

				timeDelay();
				doRead();
			}

			if(tempFreq > 88000)
			{
	
				convertToBin(tempFreq, temp);
		
				temp[1]=0x00 | (config->showSearchStr() & 0x03);
				temp[0]=0x01;	
	
				writeCard(temp);
	
				timeDelay();
				
				doRead();
			}
		}
	}

}


void tunerCard::timeDelay(void)
{
		
    	QTime new_time = QTime::currentTime();      // get the current time
    	
    	int delay = (new_time.msec() + DELAY) % 1000;
    	
    	while ( new_time.msec() != delay )  
    		new_time = QTime::currentTime();

}

/* SetMute Function sets the mute function of the FMR2 */

void tunerCard::setMute(void)
{
	data[0] = 0x02;
	data[1]&=0x7F;
	writeCard(data);
	muted=1;		
}

/* clearMute Function clears the mute function of the FMR2 */


void tunerCard::clearMute(void)
{
	data[0] = 0x00;
	data[1]&=0x7F;
	writeCard(data);
	muted=0;	
}

/* stereoToggle Function toggles the mono/stereo reception toggle of the FMR2 */

void tunerCard::stereoToggle(void)
{
	data[1] ^= 0x40;
	data[0]=0x00;
	writeCard(data);
	muted=0;

}

void tunerCard::refreshData(void)
{
	readCard(data);
}

/* loadProg loads the frequency divider ratio of the selected program number into FMR2 */

void tunerCard::loadProg(unsigned int index)
{
	setFreq(config->showFreq(index));
	progNo=index;
}

/* storeProg stores the current value of the frequency divider ratio into a user selected program number */

void tunerCard::storeProg(unsigned int index)
{
	config->setFreq(index, showFreq());
	progNo=index;
}

/* showProgNo returns the value of the currently loaded program number of the FMR2 */

int tunerCard::showProgNo(void)
{
 	return (int) progNo;	
}

